#!/usr/bin/perl -w

=head1 NAME

Debconf::Element::Web::Note - A paragraph on a form

=cut

package Debconf::Element::Web::Note;
use strict;
use base qw(Debconf::Element::Web::Text);

=head1 DESCRIPTION

This element handles a paragraph of text on a web form. It is identical to
Debconf::Element::Web::Text.

=cut

1
