#!/usr/bin/perl -w

=head1 NAME

Debconf::Element::Editor::String - Password input

=cut

package Debconf::Element::Editor::Password;
use strict;
use base qw(Debconf::Element::Editor::String);

=head1 DESCRIPTION

This is a password input.

=cut

1
