#!/usr/bin/perl -w

=head1 NAME

Debconf::Element::Teletype::Error - Show an error message to the user

=cut

package Debconf::Element::Teletype::Error;
use strict;
use base qw(Debconf::Element::Teletype::Text);

=head1 DESCRIPTION

This is an error message to output to the user.

=cut

1
